TEMPERATURE_HEX_OFF = 0x0F
TEMPERATURE_HEX_ON = 0x39
TEMPERATURE_SETPOINT_MIN = 8.0
TEMPERATURE_SETPOINT_MAX = 28.0

DEFAULT_CONFIG_LOW_BYTE = 0x02

HEX_PREFIX = "#"

REQUEST_TEMPERATURE_SETPOINT  = 0x01000000
REQUEST_TEMPERATURE_AMBIENT   = 0x02000000
REQUEST_TEMPERATURE_OFFSET    = 0x04000000
REQUEST_CONFIG                = 0x08000000
REQUEST_DATETIME              = 0x10000000
REQUEST_WINDOW_OPEN_CONFIG    = 0x20000000
REQUEST_BATTERY               = 0x40000000
REQUEST_BASE_SOFTWARE_VERSION = 0x00000200
REQUEST_WIFI_SOFTWARE_VERSION = 0x00000400
REQUEST_WIFI_SIGNAL_STRENGTH  = 0x00000800