CONNECTION_TEST = "#COMM-TEST"

REQUEST_ALL_PARAMS = "#6D000000" # Request setpoint temperature, temperature offset, configuration, window open configuration, and battery.
REQUEST_WIFI_SIGNAL = "#00000800" # Request WiFi signal strength